@@ -1,4 +1,6 @@
-from typing import Dict
+from __future__ import annotations
+
+from typing import Any
 
 from aiohttp.formdata import FormData
 
@@ -33,28 +35,35 @@ class Block(MetablockEntity):
     """Object representing a block in a space"""
 
     @property
-    def plugins(self):
+    def plugins(self) -> BlockPlugins:
         return BlockPlugins(self, "plugins")
 
     @property
-    def deployments(self):
+    def deployments(self) -> CrudComponent:
         return CrudComponent(self, "deployments")
 
-    async def config(self, *, callback=None) -> Dict:
+    async def config(self, *, callback: Any = None) -> dict:
         return await self.get(f"{self.url}/config", callback=callback)
 
-    async def ship(self, name: str, bundle: str, env="stage", *, callback=None) -> Dict:
+    async def certificate(self, *, callback: Any = None) -> dict:
+        return await self.get(f"{self.url}/certificate", callback=callback)
+
+    async def ship(
+        self, name: str, bundle: str, env: str = "stage", *, callback: Any = None
+    ) -> dict:
         data = FormData()
         data.add_field("name", name)
         data.add_field("bundle", open(bundle, "rb"), filename=bundle)
         data.add_field("env", env)
         return await self.post(f"{self.url}/deployments", data=data, callback=callback)
 
-    async def add_route(self, *, callback=None, **kwargs) -> Dict:
+    async def add_route(self, *, callback: Any = None, **kwargs: Any) -> dict:
         """Add a new route to the block"""
         return await self.post(f"{self.url}/routes", json=kwargs, callback=callback)
 
-    async def update_route(self, name: str, *, callback=None, **kwargs) -> Dict:
+    async def update_route(
+        self, name: str, *, callback: Any = None, **kwargs: Any
+    ) -> dict:
         """Update a route in the block"""
         return await self.patch(
             f"{self.url}/routes/{name}", json=kwargs, callback=callback
